@@ -24,7 +24,6 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
-
 #include <libintl.h>
 #include <locale.h>
 #include <glib/gi18n-lib.h>
@@ -34,6 +33,8 @@
 #include <string.h>
 #include <gdk/gdk.h>
 
+#include <sys/select.h>
+
 /* Settings dialog and dbus message constants */
 #include <osso-rss-feed-reader/dbus.h>
 #include <osso-rss-feed-reader/settings.h>
@@ -116,6 +117,8 @@ static void     rss_home_applet_realize      (GtkWidget *widget);
 static gboolean rss_home_applet_expose_event (GtkWidget      *widget,
                                               GdkEventExpose *event);
 
+Iphb_cb_data iphb_applet_data; // Rama added
+
 /* GConf - public */
 static void _applet_gconf_init(HomeRssApplet * applet);
 static void _applet_gconf_deinit(HomeRssApplet * applet);
@@ -152,6 +155,8 @@ static void applet_check_animation(HomeRssApplet * applet);
 
 static void _applet_display_handler(osso_display_state_t state, gpointer data);
 
+static gboolean applet_poll_iphb_fd(gpointer data);
+
 static void _size_allocate_cb (GtkWidget * widget,
                                GtkAllocation *allocation,
                                gpointer user_data);
@@ -278,6 +283,16 @@ rss_home_applet_finalize (GObject *object)
 
     dbus_error_init(&err);
 
+	/* Rama - Free up IPHB resources */
+#if 1
+	close_iphb_handle(iphb_applet_data);
+	remove_iphb_callback(iphb_applet_data.source_id);
+	iphb_applet_data.source_id = 0;
+	g_free(iphb_applet_data.user_data);
+	iphb_applet_data.user_data = NULL;
+#endif
+	/* Till Here */
+
     send_dbus_message_alarm_deletion(applet);
 
     if (priv->scroll_down_tw_timeout)
@@ -308,8 +323,10 @@ rss_home_applet_finalize (GObject *object)
         priv->timeout = 0;
       }
 
+#if 0	/* Rama - commented */
     if (priv->htimeout != 0)
       gtk_timeout_remove(priv->htimeout);
+#endif
 
     _applet_gconf_deinit(applet);
 
@@ -392,7 +409,7 @@ rss_home_applet_init (HomeRssApplet *applet)
     rss_home_applet_change_colormap (GTK_WIDGET (applet));
     create_common_view(applet);
 
-    priv->htimeout = g_timeout_add(1000, _setup_alarmd, NULL);
+    priv->htimeout = g_timeout_add(1000, _setup_alarmd, applet);
 
     hildon_helper_set_logical_color(priv->empty_label, GTK_RC_FG,
                                     GTK_STATE_NORMAL,
@@ -418,6 +435,8 @@ rss_home_applet_init (HomeRssApplet *applet)
     /* First filling of the headlines */
     _applet_set_headlines(applet);
 
+    iphb_applet_data.user_data = g_new0(HomeRssApplet, 1);
+    memcpy(iphb_applet_data.user_data, (gpointer)(applet), sizeof(HomeRssApplet));
     return;
 }
 
@@ -426,16 +445,19 @@ _setup_alarmd(gpointer data)
 {
     ULOG_DEBUG(__FUNCTION__);
 
-    if (setup_alarm_event_now(FALSE) == FALSE)    
-        {
-            ULOG_DEBUG("Must run again...\n");	    
-            return TRUE;
-        }
-    else
-        {
-            ULOG_DEBUG("Operation finished\n");	    
-            return FALSE;
-        }
+    if(iphb_applet_data.source_id == 0) {
+	    iphb_applet_data.iphb_callback = (GSourceFunc)(applet_poll_iphb_fd);
+	    iphb_applet_data.source_id = 0;
+	    //iphb_applet_data.user_data = (gpointer)data;   
+
+	    ULOG_ERR("ENTER alarm setup timeout\n");
+	    setup_alarm_event_now(TRUE, &iphb_applet_data);
+	    iphb_applet_data.source_id = setup_iphb_callback(&iphb_applet_data);
+    }
+
+    ULOG_ERR("LEAVE alarm setup timeout'n");
+    return(FALSE);
+
 }
 
 #ifdef APPLET_TEST
@@ -725,6 +747,17 @@ hw_event_handler(osso_hw_state_t * state, gpointer data)
         }
 }
 
+gint event_deletion(gpointer data)
+{
+#if 1
+	close_iphb_handle(iphb_applet_data);
+	remove_iphb_callback(iphb_applet_data.source_id);
+	iphb_applet_data.source_id = 0;
+	//iphb_applet_data.user_data = NULL;
+#endif	
+	return FALSE;
+}
+
 	static void
 _applet_display_handler(osso_display_state_t state, gpointer data)
 {
@@ -736,6 +769,10 @@ _applet_display_handler(osso_display_state_t state, gpointer data)
         {
             priv->animation = FALSE;
             stop_animation(RSS_HOME_APPLET (data));
+	    /* This part of code will disable IPHB if display goes OFF.
+	     * eventually network will not refresh if device is sleeping.
+	     * Hence disabling the code for now. - Rama */
+	    //g_timeout_add(200, event_deletion, NULL);
         }
     else if( state == OSSO_DISPLAY_ON )
         {
@@ -746,6 +783,10 @@ _applet_display_handler(osso_display_state_t state, gpointer data)
 /*                                       RSS_SETTINGS_AUTOMATIC_SCROLLING, NULL); */
 
             restart_animation(RSS_HOME_APPLET (data), ANIMATION_TO_DOWN_TIMEOUT);
+	    /* This part of code will disable IPHB if display goes OFF.
+	     * eventually network will not refresh if device is sleeping.
+	     * Hence disabling the code for now. - Rama */
+	    //priv->htimeout = g_timeout_add(200, _setup_alarmd, data);	    
         }
 }
 
@@ -843,7 +884,8 @@ _applet_handle_signal_cb(DBusConnection *
                                             OSSO_RSS_FEED_READER_ALARMDELETE_RSSSHUTDOWN_SIGNAL))
                 {
                     ULOG_DEBUG("alarm deletion");
-                    applet->priv->htimeout = g_timeout_add(1000, _setup_alarmd, NULL);
+	    			g_timeout_add(200, event_deletion, NULL);
+	    			applet->priv->htimeout = g_timeout_add(200, _setup_alarmd, user_data);	// Rama - commented
                     msg_handled = DBUS_HANDLER_RESULT_HANDLED;
                 }
         }
@@ -947,10 +989,13 @@ _applet_dbus_connect(HomeRssApplet * applet,
  *
  * @param applet the applet structure
  * @param timed if the refresh should be timed or not
+ * @param manual_refresh  if the refresh button is clicked 
+ *                    or refresh is automatic. 
+ *                        
  * @return TRUE on success, FALSE on failure
  */
 static gboolean
-_applet_dbus_refresh(HomeRssApplet * applet, gboolean timed)
+_applet_dbus_refresh(HomeRssApplet * applet, gboolean timed, gchar *manual_auto_refresh)
 {
     ULOG_DEBUG(__FUNCTION__);
 
@@ -982,6 +1027,7 @@ _applet_dbus_refresh(HomeRssApplet * applet, gboolean timed)
             dbus_message_set_auto_start(message, TRUE);
             if (!dbus_message_append_args(message,
                                           DBUS_TYPE_BOOLEAN, &timing,
+					  DBUS_TYPE_STRING,&manual_auto_refresh,
                                           DBUS_TYPE_INVALID))
                 {
                     ULOG_ERR("Could not append args to message");
@@ -1019,6 +1065,50 @@ _applet_dbus_refresh(HomeRssApplet * applet, gboolean timed)
     return TRUE;
 }
 
+static gboolean applet_poll_iphb_fd(gpointer data)
+{
+	HomeRssApplet * applet = (HomeRssApplet *)(iphb_applet_data.user_data);
+        GConfClient* gconf = gconf_client_get_default();
+        int iphb_fd = gconf_client_get_int(gconf, RSS_SETTINGS_IPHB_HANDLE_ID, NULL);	
+	fd_set fds;
+	struct timeval time_wait;
+        time_t cur_time = 0;	
+
+	memset(&time_wait, 0, sizeof(struct timeval));
+
+	time_wait.tv_sec = 60;
+	time_wait.tv_usec = 0;
+
+	if(0 >= iphb_fd) {
+		return TRUE;
+	}
+
+	if(1/*CHECK IF DISPLAY IS ON */) {
+		FD_ZERO(&fds);
+		FD_SET((unsigned int)iphb_fd, &fds);
+		if(select(iphb_fd+1,&fds,NULL,NULL,&time_wait)) {
+			if( FD_ISSET(iphb_fd,&fds) ) {
+				/* DO REFRESH */
+                                cur_time = time(NULL);
+                                gconf_client_set_int(gconf, RSS_SETTINGS_IPHB_LAST_REFRESH_TIME, cur_time, NULL);
+				
+				setup_alarm_event_now(TRUE, &iphb_applet_data);
+				iphb_applet_data.source_id = setup_iphb_callback(&iphb_applet_data);
+				
+				if (_applet_dbus_refresh(applet, FALSE,"auto") == FALSE)
+				{
+					ULOG_ERR("User requested refresh DBus message failed");
+				}
+				return FALSE;    /* Continue the call callback */
+			}
+			else {
+				return FALSE;     /* Continue the call callback */
+			}
+		}
+	}
+	return FALSE;
+}
+
 /** Calls the main application to display a selected feed
  *
  * @param applet the applet structure
@@ -1287,7 +1377,7 @@ on_menuitem_settings(GtkWidget * widget, gpointer user_data)
 {
     ULOG_DEBUG(__FUNCTION__);
 
-    execute_rss_settings(priv->osso, NULL, TRUE, TRUE);
+    execute_rss_settings(priv->osso, NULL, TRUE, TRUE, &iphb_applet_data);
 }
 #endif
 
@@ -2120,7 +2210,7 @@ refresh_button_clicked_cb (GtkWidget *self,
 
     update_refresh_button_image (applet);
 
-    if (_applet_dbus_refresh (applet, FALSE) == FALSE)
+    if (_applet_dbus_refresh (applet, FALSE,"manual") == FALSE)
         {
             ULOG_ERR ("User requested refresh DBus message failed");
         }
